import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.lang.*;
import java.net.URL;
import java.net.*;
import java.util.Vector;
import java.io.*;
import java.lang.String;

/**
 * JPanel zum Anzeigen eines Algorithmusses aus einer txt Datei.
 *
 */
public class AlgoPanel extends JPanel
{
    //Objekte frs Doublebuffering
    private BufferedImage offscreen;
    private Graphics2D bufferGraphics;
    
    private Dimension size;
    private JScrollPane aPane;

    private Vector lineStorage;
    private int lastLNr=-1;
    
    /**
     * Erzeugt eines neues Panel der angegebenen Gre.
     * @param w Breite des Panels
     * @param h Hhe des Panels
     */
    public AlgoPanel(int w, int h)
    {
	size=new Dimension(w,h);
    }
	
    /**
     * Erzeugt die Objekte frs DoubleBuffering.
     * Liest und zeichnet das TExtfile mit dem Algorithmus.
     */
    public void start(String nurl)
    {
	//image for doublebuffering
	offscreen = (BufferedImage)createImage(size.width,size.height);
	bufferGraphics = offscreen.createGraphics();
	getFromFile(nurl);
	paintText();
    }	

    /**
     * bergibt die ScrollPane, in dem das aktuelle Objekt eingebettet ist.
     * @param sPane aktive ScrollPane
     */
    public void setScrollPane(JScrollPane sPane)
     {
	aPane=sPane;
    }

    /**
     * Zeichnet den Algorithmustext auf das Hintergrundbild.
     */
    private void paintText()
    {
	int i;
	String line;
	bufferGraphics.setColor(new Color(255,255,255));
	bufferGraphics.fillRect(0,0,size.width,size.height);
	bufferGraphics.setColor(Color.black);
	bufferGraphics.setFont(new Font("Courier New", Font.PLAIN, 12));
	for(i=0;i<lineStorage.size();i++)
	{
	    line=(String)lineStorage.elementAt(i);
	    bufferGraphics.drawString(line,5,i*15+15);
	}
    }
	

    public void paintComponent(Graphics g)
    {
	super.paintComponent(g);		
	Graphics2D g2d=(Graphics2D)g;
	g2d.drawImage(offscreen,0,0,this);
    }
	
    /**
     * Liest eine Textdatei ein und legt den Inhalt zeilenweise
     * in einem String[] ab.
     * @param baseURL URL fr ein Textfile
     */
    public void getFromFile(String baseURL)
    {
	int i=0;
	String line;
	try
	{
	    URL confFile=new URL(baseURL);			
	    InputStream inStream = null;
	    inStream=confFile.openStream();
	    InputStreamReader inStReader=new InputStreamReader(inStream);
	    BufferedReader inReader = new BufferedReader(inStReader);
	    lineStorage=new Vector();
	    line=inReader.readLine();
	    while(line!=null)
	    {
		lineStorage.addElement(line);
		line=inReader.readLine();
	    }
	} catch (IOException e) {}
    }

    /**
     * Hebt eine Zeile in der Darstellung durch eine Rahmen hervor.
     * Lscht dabei vorher einen eventuell schon vorhandenen Rahmen.
     * @param lNr Zeilennummer
     */
    public void setHighlight(int lNr)
    {
	int i,w,h;
	if(lNr>=lineStorage.size()) return;
	bufferGraphics.setStroke(new BasicStroke(2));
	
	//alten Rahmen berzeichnen 
	if(lastLNr>=0)
	{
	    bufferGraphics.setColor(Color.white);
	    w=bufferGraphics.getFontMetrics().stringWidth(
			 (String)lineStorage.elementAt(lastLNr));
	    h=bufferGraphics.getFontMetrics().getHeight()-1;
	    i=lastLNr*15+3;
	    bufferGraphics.drawRect(2,i,w+20,h);
	}
	
	//neuen Rahmen zeichnen
	bufferGraphics.setColor(Color.red);
	w=bufferGraphics.getFontMetrics().stringWidth(
			 (String)lineStorage.elementAt(lNr));
	h=bufferGraphics.getFontMetrics().getHeight()-1;
	i=lNr*15+3;
	bufferGraphics.drawRect(2,i,w+20,h);
	bufferGraphics.setStroke(new BasicStroke(1));

	//die Position in der bergeordneten Scrollpane richtig setzen,
	//so dass die aktuelle Zeile sichtbar wird
	i=i-(aPane.getViewportBorderBounds().height/2);
	if(i>0)
	    aPane.getViewport().setViewPosition(new Point(0,i));

	this.repaint();
	lastLNr=lNr;
    }
}





















