import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.lang.*;
import java.net.URL;
import java.net.*;


/**
 * Panel zur Darstellung eines Bipartiten Graphen (BiPart)
 */
public class GraphPanel extends JPanel
{
    //Objekte frs Doublebuffering
    private BufferedImage offscreen;
    private Graphics2D bufferGraphics;
    
    private Dimension size;

    private BiGraph graph;
  
   
    /**
     * Erzeugt ein neues Panel.
     * @param w Breite des Panels.
     * @param h Hhe des Panels.
     * @param ngraph Graph der gezeichnet werden soll.
     */
    public GraphPanel(int w, int h, BiGraph ngraph)
    {
	size=new Dimension(w,h);
	graph=ngraph;
    }
	
    /**
     * Erzeugt die Objekte frs Doulbebuffering.
     */
    public void start()
    {
	//image for doublebuffering
	offscreen = (BufferedImage)createImage(size.width,size.height);
	bufferGraphics = offscreen.createGraphics();
    }	
	
    /** Zeichenroutine */
    public void paintComponent(Graphics g)
    {
	super.paintComponent(g);		
	Graphics2D g2d=(Graphics2D)g;
	Vertex toPaint, eTarget;
	Edge pEdge;
	int i;
	String tmpStr;
	
	size=this.getSize();

	// neue y-Koordinaten fr alle Knoten setzen
	// Das GraphPanel befindet sich innerhalb einer JSlidePane
	// somit ist die Hhe variabel
	graph.rewindAll();
	toPaint=graph.getNextAVertex();
	while(toPaint!=null)
	{
	    toPaint.pos_y=size.height*3/4;
	    toPaint=graph.getNextAVertex();
	}
	toPaint=graph.getNextBVertex();
	while(toPaint!=null)
	{
	    toPaint.pos_y=size.height*1/4;
	    toPaint=graph.getNextBVertex();
	}


	//Hintergrund lschen
	bufferGraphics.setBackground(new Color(0,0,0));
	bufferGraphics.clearRect(0,0,size.width,size.height);
	bufferGraphics.setColor(new Color(255,255,255));
	bufferGraphics.fillRect(2,2,size.width-4,size.height-4);

	//Knotenbezeichnungen
	bufferGraphics.setColor(Color.black);
	bufferGraphics.setFont(new Font("SansSerif",Font.BOLD,16));
	bufferGraphics.drawString("Sinks", 5,size.height-10);
	bufferGraphics.drawString("Sources", 5, 20);
	bufferGraphics.setFont(new Font("SansSerif", Font.PLAIN, 11));

	//Kanten zeichnen
	graph.rewindAll();
	pEdge=graph.getNextEdge();
	while(pEdge!=null)
	{
	    toPaint=pEdge.getAVertex();
	    eTarget=pEdge.getBVertex();
	    bufferGraphics.setColor(pEdge.getColor());
	    bufferGraphics.setStroke(new BasicStroke(pEdge.getThickness()));
	    bufferGraphics.drawLine(toPaint.pos_x, toPaint.pos_y,
				    eTarget.pos_x, eTarget.pos_y);
	    pEdge=graph.getNextEdge();
	}
	bufferGraphics.setStroke(new BasicStroke(1));

	//Knoten zeichnen
	graph.rewindAll();
	toPaint=graph.getNextAVertex();
	while(toPaint!=null)
	{
	    i=toPaint.radius;
	    bufferGraphics.setColor(toPaint.getColor());
	    bufferGraphics.fillOval(toPaint.pos_x-i, toPaint.pos_y-i,i+i,i+i);
	    if(toPaint.edgeEdit) bufferGraphics.setColor(Color.RED);
	    tmpStr=new String(new Integer(graph.getCurrentA()).toString());
	    bufferGraphics.drawString(tmpStr,
		  toPaint.pos_x-(bufferGraphics.getFontMetrics().stringWidth(
		  tmpStr)/2),toPaint.pos_y+i+12);	      
	    toPaint=graph.getNextAVertex();
	}

	bufferGraphics.setColor(Color.BLUE);
	toPaint=graph.getNextBVertex();
	while(toPaint!=null)
	{
	    i=toPaint.radius;
	    bufferGraphics.setColor(toPaint.getColor());
	    bufferGraphics.fillOval(toPaint.pos_x-i, toPaint.pos_y-i,i+i,i+i);
	    if(toPaint.edgeEdit) bufferGraphics.setColor(Color.BLUE);;
	    tmpStr=new String(new Integer(graph.getCurrentB()).toString());
	    bufferGraphics.drawString(tmpStr,
		  toPaint.pos_x-(bufferGraphics.getFontMetrics().stringWidth(
		  tmpStr)/2), toPaint.pos_y-i-5);
	    toPaint=graph.getNextBVertex();
	}

	//Bild einblenden
	g2d.drawImage(offscreen,0,0,this);
    }
	

    /**
     * Neuen Graph bergeben.
     * @param ng Bipartiter Graph
     */
    public void setGraph(BiGraph ng)
    {
	graph=ng;
    } 
    
    /**
     * Ermittelt anhandt der Farbe eines Pixels, ob an dieser Stelle schon
     * etwas gezeichnet wurde.
     * @param x X-Koordinate
     * @param y Y-Koordinate
     * @return true, wenn noch Keine Grafik existiert, false sonst
     */
    public boolean placeIsEmpty(int x,int y)
    {
	int cx, cy;
	for(cy=y-10;cy<y+10;cy++)
	{
	    for(cx=x-10;cx<x+10;cx++)
	    {
		if(offscreen.getRGB(cx,cy)!=Color.WHITE.getRGB()) return(false);
	    }
	}
	return(true);
    }
	
}





















