import java.util.Vector;
import java.awt.*;

/**
 * Algorithmus fr das Matching in Bipartiten Graphen.
 */
public class MatchAlg1 implements Runnable
{
    private Thread mainThread;
    private final static int CNBiGrInvVtx=-1;
    private final static int CNBiGrInvVtxIdx=-1;
    private BiGraph bg;
    private int noFull,runState, sleepTime=300,retValue;
    private boolean started=false, stepping;
    private BiPartGraph father;

    //-------- Variablen, die im eigentlichen Algorithmus verwendet werden
    private int result[], usedBy[], numSrcs, numSnks, snkC[], srcL[],
	matches=0, reserve, snk0,snk,src,nxt,i,n;
    private Vertex locSnks[];
    private Vector snkX;
    //--------

    public final static int FAST_FORWARD=0, RUN=1, STEPPING=2;
    private final static int DEFAULT_ST=300;


    /**
     * @param bpg BiPartGraph Applet, der dieses Objekt erzeugt
     */
    public MatchAlg1(BiPartGraph bpg)
    {
	father=bpg;
    }

    /**
     * Setzt die Sleeptime fr den RUN Modus.
     * @param nst neue Sleeptime in ms
     */
    public void setSleeptime(int nst)
    {
	sleepTime=nst;
    }

    /**
     * Ermittelt, ob der Algorithmus gerade luft.
     * @return true, wenn der Algorithmus luft, false sonst
     */
    public boolean isRunning()
    {
	return(started);
    }

    /**
     * Erzeugt den Hauptthread und startet den Algorithmus.
     * @param bgN Bipartiter Graph fr das Matching
     * @param noFullN volles Matching
     */
    public void getMatch(BiGraph bgN, int noFullN)
    {
	bg=bgN;
	noFull=noFullN;
	runState=STEPPING;
	started=true;
	mainThread=new Thread(this);
	mainThread.start();
    }
    
    /**
     * Fhrt einen Schritt im Algorithmus aus.
     */
    public void doStep()
    {
	if((started) && (runState==STEPPING))
	{
	    stepping=false;
	    synchronized (mainThread)
	    {
		mainThread.notify();
	    }
	  
	}
    }
    
    /**
     * Starte den RUN-Modus.
     */
    public void doStart()
    {
	if(runState==STEPPING || runState==FAST_FORWARD)
	{
	    if(runState==STEPPING) sleepTime=DEFAULT_ST;
	    System.out.println("start");
	    if(runState==STEPPING) runState=RUN;
	    stepping=false;
	    synchronized (mainThread)
	    {
		mainThread.notify();
	    }
	    father.updateData(collectData());
	}
    }

    /**
     * Hlt die Ausfhrung an.
     */
    public void doStop()
    {
	runState=STEPPING;
    }

    /**
     * Startet den FastForward Modus.
     */
    public void doFF()
    {
	System.out.println("FF");
	sleepTime=0;
	runState=FAST_FORWARD;
	doStart();
	
    }

    /**
     * StateMachine fr die Algorithmus-Abarbeitung.
     * @param line Zeile im Quelltext, an der sich der Algorithmus befindet
     */
    private void waitOrGo(int line)
    {
	switch(runState)
	{
  	    case STEPPING:
		father.updateGraphPanel();
		father.setHighlight(line);
		father.updateData(collectData());
		stepping=true;
		System.out.println("step");
		while(stepping)
		{
		    synchronized (mainThread)
		    {
			try
			{
			    mainThread.wait();
			} catch(InterruptedException e) {}
		    }
		}
		break;
 	    case FAST_FORWARD:
		System.out.print("f");
		try
		{
		    mainThread.sleep(sleepTime);
		} catch (InterruptedException e) {System.out.println("ire");}
		break;
	    case RUN:
		father.updateGraphPanel();
		father.setHighlight(line);
		father.updateData(collectData());
		System.out.print(".");
		try
		{
		    mainThread.sleep(sleepTime);
		} catch (InterruptedException e) {System.out.println("ire");}
		
		break;
	}
    }

    /**
     * Matching Thread
     */
    public void run()
    {
	System.out.println("Machting ...");
	int j;
	Edge edgeT;
	Vector ePath=new Vector();


	numSnks=bg.getACount();
	numSrcs=bg.getBCount();

	System.out.println("snks: "+numSnks+"  srcs: "+numSrcs);
	result=new int[numSrcs];
	for(j=0;j<numSrcs;j++) result[j]=CNBiGrInvVtx;
	usedBy=new int[numSnks];
	for(j=0;j<numSnks;j++) usedBy[j]=CNBiGrInvVtx;

	if (noFull == 0 && numSnks < numSrcs)
	{
	    retValue=0;
	    matchingDone(1);
	    return;   // no full source match possible
	}

	if (numSrcs == 0)	// no src -> always a match
	{
	    retValue=1;
	    matchingDone(2);
	    return;   // no full source match possible
	}

	locSnks=new Vertex[numSnks];
	for(j=0;j<numSnks;j++) locSnks[j]=new Vertex(j,0,0,CNBiGrInvVtx);

	snkX=new Vector();
	snkC=new int[numSnks];
	for(j=0;j<numSnks;j++) snkC[j]=CNBiGrInvVtxIdx;
	srcL=new int[numSrcs];
	for(j=0;j<numSrcs;j++) srcL[j]=CNBiGrInvVtxIdx;

	matches=0;
	reserve=numSnks-numSrcs;
	src=0;

	System.out.println("starting alg ..");
	waitOrGo(24);
	for (snk0 = 0; snk0 < numSnks; snk0++)
	{
	    waitOrGo(27);
	    snkX.clear();
	    nxt=0;
	    snk=snk0;
	    waitOrGo(33);
	    n=bg.getVertexAByNumber(snk).getNumberOfEdges();
	    bg.resetAllAColor();
	    bg.getVertexAByNumber(snk).setMark(2);
	    waitOrGo(34);
	    for(i=1;i<=n;i++)
	    {
		waitOrGo(35);
		//src=bg.getVertexAByNumber(snk).getEdgeTo(n-i);
		edgeT=bg.getVertexAByNumber(snk).getEdgeByNr(n-i);
		src=edgeT.getBVertex().getNr();
		edgeT.setColor(Color.CYAN);
		ePath.addElement(edgeT);
		bg.resetAllBColor();
		bg.getVertexBByNumber(src).setMark(2);
		waitOrGo(36);
		nxt=result[src];
		waitOrGo(37);
		if (nxt == CNBiGrInvVtxIdx) // path end found (src is the end)
		{
		    clearEdgePath(ePath);
		    break;
		}
		else
		{
		    snkC[nxt]=snk;
		    snkX.addElement(new Integer(nxt));
		}
		waitOrGo(43);
		srcL[src]=snk0;
		//		waitOrGo();
	    }
	 
	   
	    waitOrGo(46);
	    while((snkX.size()!=0) && (nxt!=CNBiGrInvVtxIdx))
	    {
		snk=((Integer)snkX.lastElement()).intValue();
		waitOrGo(47);
		snkX.removeElementAt(snkX.size()-1);
		waitOrGo(48);
		//org: n = locSnks[snk].getNumEdgs();
		n=locSnks[snk].getNumberOfEdges();
		waitOrGo(49);
		for(i=1;i<=n;i++)
		{
		    //org: src = locSnks[snk].getEdgTo(n-i);
		    //  src=locSnks[snk].getEdgeTo(n-1);
		    edgeT=locSnks[snk].getEdgeByNr(n-i);
		    edgeT.setColor(Color.CYAN);
		    ePath.addElement(edgeT);
		    src=edgeT.getBVertex().getNr();
		    waitOrGo(53);
		    if(srcL[src]==snk0) continue;
		    srcL[src]=snk0;
		    waitOrGo(56);
		    nxt=result[src];
		    waitOrGo(57);
		    if(nxt==CNBiGrInvVtxIdx) break;
		    else
		    {
			snkC[nxt]=snk;
			waitOrGo(61);
			snkX.addElement(new Integer(nxt));
		    }
		}

	    }

	    clearEdgePath(ePath);

	    waitOrGo(65);
	    if(nxt!=CNBiGrInvVtxIdx)
	    {
		if((noFull==0) && (reserve-- ==0))
		{
		    retValue=0;
		    matchingDone(0);
		    return;
		}
		continue;
	    }
	    do
	    {
		nxt=usedBy[snk];
		waitOrGo(74);
		usedBy[snk]=src;
		waitOrGo(75);
		result[src]=snk;
		setResult();
		waitOrGo(76);
		src=nxt;
		waitOrGo(77);
		snk=snkC[snk];
	    } while(nxt!=CNBiGrInvVtxIdx);

	    if (++matches == numSrcs)
	    {
		retValue=1;
		matchingDone(3);
		return;
	    }

	    //org: locSnks[snk0]=snks[snk0];;
	    locSnks[snk0]=bg.getVertexAByNumber(snk0);
	    
	}
	waitOrGo(87);
	retValue=noFull;
	matchingDone(4);
	return;
    }

    /**
     * Aktualisiert die Graphendarstellung anhand von result[]
     */
    private void setResult()
    {
	int j;
	Edge tEdge;
	//	started=false;
	bg.resetAllEdgeAttr();
	for(j=0;j<numSrcs;j++)
	{
	    bg.rewindAll();
	    tEdge=bg.getNextEdge();
	    while(tEdge!=null)
	    {
		if(tEdge.getBVertex().getNr()==j)
		    if(tEdge.getAVertex().getNr()==result[j])
			tEdge.setThickness(3);
		tEdge=bg.getNextEdge();
	    }
	}
    }

    /**
     * Wird aufgerufen, wenn der Alorithmus beendet wurde.
     * @param result Endcode
     */
    private void matchingDone(int result)
    {
	setResult();
	started=false;
	father.updateData(collectData());
	father.matchingDone(result);
	bg.resetAllAColor();
	bg.resetAllBColor();
    }

    /**
     * Erstellt die HTML-Tabellen mit den Datenstrukturen
     * @return HTML String
     */
    private String collectData()
    {
	int j;
	String retStr=new String("<html><body>");
	
	retStr=retStr.concat("<table border=\"1\">");

	retStr=retStr.concat("<tr><th>Src Nr</th>");
	for(j=0;j<numSrcs;j++)
	    retStr=retStr.concat("<th>"+new Integer(j).toString()+"</th>");

	retStr=retStr.concat("</tr><tr><th>result[]</th>");
	for(j=0;j<numSrcs;j++)
	    retStr=retStr.concat("<td>"+new Integer(result[j]).toString()+
				 "</td>");
	retStr=retStr.concat("</tr><tr><th>srcL[]</th>");
	for(j=0;j<numSrcs;j++)
	    retStr=retStr.concat("<td>"+new Integer(srcL[j]).toString()+
				 "</td>");
	retStr=retStr.concat("</tr></table><br>");


	retStr=retStr.concat("<table border=\"1\">");
	retStr=retStr.concat("<tr><th>Snk Nr</th>");
	for(j=0;j<numSnks;j++)
	    retStr=retStr.concat("<th>"+new Integer(j).toString()+"</th>");

	retStr=retStr.concat("</tr><tr><th>usedBy[]</th>");
	for(j=0;j<numSnks;j++)
	    retStr= retStr.concat("<td>"+new Integer(usedBy[j]).toString()+
				  "</td>");


	retStr=retStr.concat("</tr><tr><th>snkC[]</th>");
	for(j=0;j<numSnks;j++)
	    retStr= retStr.concat("<td>"+new Integer(snkC[j]).toString()+
				  "</td>");
	retStr=retStr.concat("</tr></table>");

	retStr=retStr.concat("<br><table border=\"1\">");
	retStr=retStr.concat("<tr><th>snkX[]</th>");
	for(j=0;j<snkX.size();j++)
	    retStr=retStr.concat("<td>"+
				 ((Integer)snkX.elementAt(j)).toString()+
				 "</td>");
	retStr=retStr.concat("</tr></table>");


	retStr=retStr.concat("<br><table border=\"1\">");


	retStr=retStr.concat("<tr><th>matches</th><td>"+
			     new Integer(matches).toString()+"</td></tr>");
	retStr=retStr.concat("<tr><th>reserve</th><td>"+
			     new Integer(reserve).toString()+"</td></tr>");
	retStr=retStr.concat("<tr><th>snk0</th><td>"+
			     new Integer(snk0).toString()+"</td></tr>");
	retStr=retStr.concat("<tr><th>snk</th><td>"+
			     new Integer(snk).toString()+"</td></tr>");
	retStr=retStr.concat("<tr><th>src</th><td>"+
			     new Integer(src).toString()+"</td></tr>");
	retStr=retStr.concat("<tr><th>nxt</th><td>"+
			     new Integer(nxt).toString()+"</td></tr>");
	retStr=retStr.concat("<tr><th>i</th><td>"+
			     new Integer(i).toString()+"</td></tr>");
	retStr=retStr.concat("<tr><th>n</th><td>"+
			     new Integer(n).toString()+"</td></tr>");
	retStr=retStr.concat("</table>");

	retStr=retStr.concat("</body></html>");
	/*
	  todo:	  private Vertex locSnks[];
	*/
	//	System.out.println(retStr);
	return retStr;
    }

    public void printResult()
    {
	int j;
	System.out.println("Match:");
	System.out.println("Result:");
	for(j=0;j<numSrcs;j++)
	    System.out.print(result[j]+" ");
	System.out.println(" ");
	System.out.println("usedBy:");
	for(j=0;j<numSnks;j++)
	    System.out.print(usedBy[j]+" ");
	System.out.println(" ");
    }

    /**
     * Demarktiert und baut eine Vector aus Kanten ab.
     * @param e Vector aus Kanten
     */
    private void clearEdgePath(Vector e)
    {
	int j;
	if(e.size()==0) return;
	for(j=e.size()-1;j>0;j--);
	{
	    ((Edge)e.elementAt(j)).setColor(Color.BLACK);
	    e.removeElementAt(j);
	}
    }
}







