import java.util.Vector;
import java.awt.*;

/**
 * Knoten eines Graphen.
 */
public class Vertex
{
    protected int pos_x, pos_y, radius;
    protected boolean edgeEdit;

    private Vector edgeTo;
    
    private boolean tag;
    private int eCounter, myNr;
    private int myCurrentColor=-1;
    private Color myColors[];

    /**
     * @param nr Nummer des Knotens
     * @param x X-Koordinate des Knotes
     * @param y Y-Koordinate des Knotens
     * @param r Radius des Knotens
     */
    Vertex(int nr, int x, int y, int r)
    {
	myNr=nr;
	pos_x=x;
	pos_y=y;
	radius=r;
	edgeTo=new Vector();
	eCounter=0;
    }

    /**
     * @param nr Nummer des Knotens
     * @param x X-Koordinate des Knotes
     * @param y Y-Koordinate des Knotens
     * @param r Radius des Knotens
     * @param nC[] Default Farben
     */
    Vertex(int nr, int x, int y, int r, Color nC[])
    {
	this(nr,x,y,r);
	myColors=nC;
	myCurrentColor=0;
    }

    /**
     * Markiert den Knoten.
     */
    public void tagEm()
    {
	tag=true;
    }
    
    /**
     * Demarkiert den Knoten.
     */
    public void unTagEm()
    {
	tag=false;
    }
    
    /**
     * @return true, wenn der Knoten markiert ist, false sonst
     */
    public boolean isTagged()
    {
	return(tag);
    }

    /**
     * Fgt eine Kante diesem Knoten hinzu.
     * @param e neue Kante
     */
    public void addEdge(Edge e)
    {
	int i;
	for(i=0;i<edgeTo.size();i++)
	    if((Edge)edgeTo.elementAt(i)==e) return;
	for(i=0;i<edgeTo.size();i++)
	    if(((Edge)edgeTo.elementAt(i)).getBVertex().getNr()>=
	       e.getBVertex().getNr()) break;
	if(i<edgeTo.size()) edgeTo.add(i,e);
	else edgeTo.addElement(e);
    }

    /**
     * Setzt den Abfragestatus zurck.
     */
    public void rewind()
    {
	eCounter=0;
    }


    /**
     * @return Nummer des Knotens
     */
    public int getNr()
    {
	return(myNr);
    }

    /**
     * Liefert die Nummer des nchsten Knotens, der mit diesem durch eine Kante
     * verbunden ist.
     * @return Knotennummer des Zielknotens
     */
    public int getNextEdgeTarget()
    {
	if(eCounter<edgeTo.size())
	{
	    eCounter++;
	    Edge tmp=(Edge)edgeTo.elementAt(eCounter-1);
	    Vertex vtmp=tmp.getBVertex();
	    return(vtmp.getNr());
	}
	return(-1);
    }
    

    /**
     * Ermittelt, ob ein Punkt in den Bereich dieses Knotens fllt.
     * @param x X-Koordinate
     * @param y Y-Koordinate
     * @return true, wenn dieser Knoten getroffen wurde, false sonst
     */
    public boolean thatMeansMe(int x, int y)
    {
	if((Math.sqrt(Math.pow((x-pos_x),2)+Math.pow((y-pos_y),2)))<radius)
	    return(true);
	else
	    return(false);
    }

    /**
     * Toggelt den EdgeEdit-Status und die aktuelle Farbe.
     */
    public void setEdgeEdit()
    {
	edgeEdit=!edgeEdit;
	if(edgeEdit) myCurrentColor=1;
	else myCurrentColor=0;
    }

    /**
     * @return Anzahl der Kanten dieses Knotens
     */
    public int getNumberOfEdges()
    {
	return(edgeTo.size());
    }
  
    /**
     * Ermittelt den Zielknoten einer bestimmten Kante.
     * @param enr Nummer der Kante
     * @return Nummer des Zielnotens der Kante
     */
    public int getEdgeEndVertexNr(int enr)
     {
 	if(enr>edgeTo.size()-1) return -1;
 	Edge tmp=(Edge)edgeTo.elementAt(enr);
 	Vertex vtmp=tmp.getBVertex();
 	return(vtmp.getNr());
     }

    /**
     * Liefert die Kante anhand ihrer Nummer.
     * @param enr Kantennummer
     * @return Kante
     */
    public Edge getEdgeByNr(int enr)
    {
	if(enr>edgeTo.size()-1) return null;
	return((Edge)edgeTo.elementAt(enr));
    }

    /**
     * Liefert die aktuelle Farbe
     * @return aktuelle Farbe
     */
    public Color getColor()
    {
	if(myColors==null) return(Color.BLACK);
	if(myCurrentColor>=myColors.length) return(Color.BLACK);
	return(myColors[myCurrentColor]);
    }

    /**
     * Setzt eine Farbe fr diesen Knoten.
     * @param m Nummer der neuen Farbe
     */
    public void setMark(int m)
    {
	myCurrentColor=m;
    }

}

