int CNBipartiteGraph::calcFullSrcMatch_IIIsortedSnks(CNBiGrMtc& mtc, int noFull)
{
    CNBiGrVtxIdxVec& result = mtc.getSrcToSnk();
    CNBiGrVtxIdxVec& usedBy = mtc.getSnkToSrc();

    CNBiGrVtxIdx numSnks = getNumSnks();
    CNBiGrVtxIdx numSrcs = getNumSrcs();
    if (noFull == 0 && numSnks < numSrcs)
       return 0;   // no full source match possible

    result.resize(numSrcs); result.setAllToE0();    // stores actual match
    usedBy.resize(numSnks); usedBy.setAllToE0();    // stores actual match 
    if (getNumSrcs() == 0)	// no src -> always a match
       return 1;
    
    CNBiGrVtxVecFixed locSnks(numSnks,CNBiGrInvVtx);  // local sinks & edges
    CNBiGrVtxIdxBufFixed snkX(numSnks);  // snks to expand
    CNBiGrVtxIdxVecFixed snkC(numSnks,CNBiGrInvVtxIdx);  // to track paths
    CNBiGrVtxIdxVecFixed srcL(numSrcs,CNBiGrInvVtxIdx);  // layer info

    CNBiGrVtxIdx matches  = 0; // actual number of matches
    CNBiGrVtxIdx reserve  = numSnks - numSrcs; // max number of unmatched snks
    // reserve is only used in full matches !!!
    CNBiGrVtxIdx snk0, snk, src, nxt, i, n;
    for (snk0 = 0; snk0 < numSnks; snk0++){
	
        // search a path from snk0 to an unused source
        snkX.clear();
        nxt = 0;    // initially a valid idx

        // generate first generation (we prefer special test and do not
        // insert snk0 into locSnks before we know that it is ok)
        snk = snk0;
        n = snks[snk].getNumEdgs();
        for(i=1;i<=n;i++){ // orig: 0..<n  @@@1
            src = snks[snk].getEdgTo(n-i); // @@@1
            nxt = result[src];
            if (nxt == CNBiGrInvVtxIdx) // path end found (src is the end)
               break;
               else{ // nxt is index of next possible snk in path
                     snkC[nxt] = snk;	    // we came from snk0
                     snkX.append(nxt);	    // expand this snk too
               }
               srcL[src] = snk0; // now in layer for processing snk0
        }

        while(snkX.getLen() != 0   &&   nxt != CNBiGrInvVtxIdx){
            snk = snkX.pop();
            n = locSnks[snk].getNumEdgs();
            for(i=1;i<=n;i++){ // orig: 0..<n  @@@1
                // In second generation one of these srcs is src from which we
                // reached this snk, don't worry it's detected fast via srcL.
                src = locSnks[snk].getEdgTo(n-i); // @@@1
                if (srcL[src] == snk0)
                   continue;	// this src is already checked
                srcL[src] = snk0; // now in layer for processing snk0
                nxt = result[src];
                if (nxt == CNBiGrInvVtxIdx) // path end found (src is the end)
                   break;
                else{ // nxt is index of next possible snk in path
                    snkC[nxt] = snk;	    // we came from snk
                    snkX.append(nxt);	    // expand this snk too
                }
            }
        }
        if (nxt != CNBiGrInvVtxIdx){ // snk not usable without making other unused
           if (noFull == 0 && reserve-- == 0 ) // test reserve for full match
              return 0; // # no reserve for a FULL match
           continue;	// this sink could not be used -> try next sink
        }

        // switch the path beginning from src, snk is first sink to switch
        do {
            nxt = usedBy[snk];	// next src to process
            usedBy[snk] = src;
            result[src] = snk;
            src = nxt;
            snk = snkC[snk];
        } while(nxt != CNBiGrInvVtxIdx);

        // this sink could be used, test if full match reached now
        if (++matches == numSrcs)
           return 1; // each full match is a max match too

        // store new sink and its edges in locSnks
        locSnks[snk0]=snks[snk0];
    }
    return noFull; // noFull (max) matches are OK here, full matches NOT
}

